#!/bin/bash
##
## Rewrite of restart_truetouch.sh to include logging and X failure shutdowns.
myName=$( basename $0 )

# Nice try, but it seems this logger(1) ignores --id= or $$ isn't what is should be
# The local journalctl prints PIDs of the logger processes.
# /var/log/syslog seems to work as expected.
myID="$$"

# Force to the console display.  Can probably be omitted as it's the expected default.
#   ...but we're explicitly using :0 with the xrandr(1).
DISPLAY=:0


###
# Procedures
###

# using the strike value for a sleep call later in the procedure.
# using print "condition" to determine good (<100) vs bad (>=100).
# A "return 1" caused the entire script to fail!
isup-eDP () {

  for strike in 3 3 0 ; do
    if /usr/bin/xrandr --display :0 | /bin/grep -q '^eDP.. connected primary 1920x1080' ; then
      printf '0\n'
      /usr/bin/logger -tTouchRestart -p user.info "INFO: eDP connected primary 1920x1080. Good condition 0"
      return 0
    elif /usr/bin/xrandr --display :0 | /bin/grep -q '^eDP.. connected primary ' ; then
      printf '1\n'
      /usr/bin/logger -tTouchRestart -p user.info "INFO: eDP connected primary. Good condition 1"
      return 0
    elif /usr/bin/xrandr --display :0 | /bin/grep -q '^eDP.. connected ' ; then
      printf '2\n'
      /usr/bin/logger -tTouchRestart -p user.info "INFO: eDP connected. Good condition 2"
      return 0
    elif /usr/bin/xrandr --display :0 --listactivemonitors | /bin/grep -q '^ 0: +*eDP-1 1920/344x1080/194+0+0  eDP-1' ; then
      printf '3\n'
      /usr/bin/logger -tTouchRestart -p user.info "INFO: eDP from listactivemonitors. Good condition 3"
      return 0
    elif /usr/bin/xrandr --display :0 --listmonitors | /bin/grep -q '^ 0: +*eDP-1 1920/344x1080/194+0+0  eDP-1' ; then
      printf '4\n'
      /usr/bin/logger -tTouchRestart -p user.info "INFO: eDP from listmonitors. Good condition 4"
      return 0
    fi

    /usr/bin/logger -tTouchRestart -p user.err --id=$myID "ERROR: cannot find eDP, now counting strikes."
    /bin/sleep $strike

  done

  # "for strike do" loop has completed 3 iterations.  You're out!
  /usr/bin/logger -tTouchRestart -p user.crit "CRIT: BlackScreenCaught: cannot find the eDP port. Three strikes."
  printf '100\n'
}

###
# Main
###

/usr/bin/logger -p user.info -tTouchRestart --id=$myID "INFO: Initializing Touch app restart monitor. Process ID ${myID}"

while true ; do

  /usr/bin/logger -p user.info -tTouchRestart --id=$myID "INFO: checking eDP status."

  myCondition="$( isup-eDP )"

  if [[ $myCondition -lt 100 ]] ; then
    /usr/bin/logger -p user.info -tTouchRestart --id=$myID "INFO: HelloTouchScreen: isup-eDP reports the eDP is up. Condition is $myCondition"
  else
    /usr/bin/logger -p user.emerg -tTouchRestart --id=$myID "EMERGENCY: BlackScreenCaught: isup-eDP reports the eDP is down. Condition is $myCondition"

    # Save xrandr information!!!
    /usr/bin/xrandr --display :0  | /usr/bin/logger -tTouchRestart -p user.crit --id=$myID

    # This is where we kill Touch and its keepalive monitor, if necessary.
    # This process controlls everything!
    # ps -ef | grep [/]usr/sbin/lightdm
    /usr/bin/logger -p user.emerg -tTouchRestart --id=$myID "EMERGENCY: BlackScreenCaught: Stopping X Display Manager."
    /usr/bin/sudo /bin/systemctl stop lightdm

    # Then sleep for a bit
    sleep 5

    # Send message to /dev/ttyEVERYTHING.  -p x.emerg does this, too.
    # The true user is in the tty group, no sudo(8) needed to write to /dev/tty<num>.
    # If it's a false alarm, the user will see this on the screen and, hopefully, report it.
    # YIKES!  myXrandr not set!
    for i in $( ls -1 /dev/tty[0-9]* ) ; do
      /bin/echo "$myXrandr" >$i 2>/dev/null
      printf '\n\n\n\n%-20sDisplay Connector: Possible eDP-1 Hardware Failure.\n' "${i}:" >$i 2>/dev/null
      printf '                    If a power cycle does not fix correct this problem, contact technical support.\n\n' $i >$i 2>/dev/null
    done

    # Here we bail out of the program.  It *may* be difficult to detect a "stop Touch, start Touch" loop.
    exit 1
  fi

  # This *should* be outsiee the "while true" loop!
  cd /usr/lib/truefitness/truetouch
  if [[ $? != 0 ]] ; then
    /usr/bin/logger -p user.crit -tTouchRestart --id=$myID "FAIL: Cannot find /usr/lib/truefitness/truetouch"
    exit 1
  fi

  # If it's not running, launch the application
  if ! pidof /usr/lib/truefitness/truetouch/Touch >/dev/null 2>&1 ; then
    /usr/bin/logger -p user.info -tTouchRestart --id=$myID "INFO: Touch app start next."
    /usr/lib/truefitness/truetouch/Touch --fullscreen --hidecursor 2>&1 | tee -a /tmp/truetouch.log >/dev/tty8
    STATUS=$?
    /usr/bin/logger -p user.alert -tTouchRestart --id=$myID "ALERT: Restart dropthrough.  Checking status table with STATUS=$STATUS"
    RESTART=0
    case $STATUS in
      139) ;& # SIGSEGV
      137) ;& # SIGKILL
      132) ;& # SIGILL
      134) ;& # SIGABRT
      136)    # SIGFPE
              RESTART=1
              ;;
      *)
              RESTART=0
              ;;
    esac
    if [ $RESTART -eq 0 ]; then
      /usr/bin/logger -p user.emerg -tTouchRestart --id=$myID "EMERGENCY: No restart attempted. Restart=${RESTART} Status=${STATUS}"
      exit $STATUS
    fi

    echo Truetouch exited with status $STATUS. Restarting... >/dev/tty8
    /usr/bin/logger -p user.alert -tTouchRestart --id=$myID "ALERT: Truetouch exited with status $STATUS. Restart attempt beginning."
    #touch /tmp/truetouch.crash
    killall AlphaKeypad 2>&1 | /usr/bin/logger -p user.alert -tTouchRestart --id=$myID
    killall WebBrowser  2>&1 | /usr/bin/logger -p user.alert -tTouchRestart --id=$myID

  else
    /usr/bin/logger -p user.alert -tTouchRestart --id=$myID "ALERT: $myName started, but Touch app already running."
    sleep 100
  fi

done

/usr/bin/logger -p user.notice -tTouchRestart --id=$myID "NOTICE: Outside of loop. This should not be. The restart monitor has stopped."
